/* Project name:
     MP3 Demo
 * Copyright:
     (c) Mikroelektronika, 2013.
* Description:
*     This project demonstrates communication with VS1053B mp3 codec.
*     Program reads one mp3 file from MMC and sends it to VS1053B for decoding
*     and playing.
* Test configuration:
*     MCU:             STM32F407ZG
*                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/DM00037051.pdf
*     Dev.Board:       Mikromedia_Plus_for_STM32_ARM ac:Media_Plus
*                      http://www.mikroe.com/mikromedia/plus/stm32-m4/
*     Oscillator:      150000000 Hz
*     SW:              mikroC PRO for ARM
*                      http://www.mikroe.com/mikroc/arm/
* NOTES:
*     - Make sure that MMC card is properly formatted (to FAT16 or just FAT)
*       before testing it on this example.
*     - Make sure that MMC card contains appropriate mp3 file ( 1.mp3 to 25.mp3).
**************************************************************************************************/

char mp3_filename[25][7] ={ "1.mp3","2.mp3","3.mp3","4.mp3","5.mp3","6.mp3","7.mp3","8.mp3","9.mp3","10.mp3","11.mp3","12.mp3","13.mp3","14.mp3","15.mp3","16.mp3","17.mp3","18.mp3","19.mp3","20.mp3","21.mp3","22.mp3","23.mp3","24.mp3","25.mp3"};
unsigned int roma;
#include <built_in.h>
#include "MP3_Test.h"
#include "resources.h"

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOF_ODR.B14;
sbit TFT_RS at GPIOF_ODR.B15;
sbit TFT_CS at GPIOF_ODR.B13;
sbit TFT_RD at GPIOF_ODR.B12;
sbit TFT_WR at GPIOF_ODR.B11;
sbit TFT_BLED at GPIOF_ODR.B10;

/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
// MP3 Codec Connections
sbit MP3_CS                    at GPIOD_ODR.B11;
sbit MP3_RST                   at GPIOD_ODR.B8;
sbit BSYNC                     at GPIOD_ODR.B10;
sbit DREQ                      at GPIOD_IDR.B9;

/**************************************************************************************************
* Functions for TFT_Set_Active
**************************************************************************************************/
void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  unsigned int temp;
  temp = GPIOE_ODR;
  temp &= 0x00FF;
  GPIOE_ODR = temp | (_hi << 8);
  temp = GPIOG_ODR;
  temp &= 0xFF00;
  GPIOG_ODR = temp | _lo;
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Write_to_Data_Lines(Hi(_data), Lo(_data));
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

/**************************************************************************************************
* Function MCU_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MCU
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
sbit Mmc_Chip_Select at GPIOD_ODR.B3;

void InitMCU(){

  // MP3_CS, MP3_RST and BSYNC are outputs
  GPIO_Config(&GPIOD_BASE, _GPIO_PINMASK_8 | _GPIO_PINMASK_10 | _GPIO_PINMASK_11, _GPIO_CFG_DIGITAL_OUTPUT);
  // DREQ is input
  GPIO_Config(&GPIOD_BASE, _GPIO_PINMASK_9, _GPIO_CFG_DIGITAL_INPUT);

  // Init TFT
  TFT_Set_Default_Mode();
  GPIO_Digital_Output(&GPIOG_BASE, 0x00FF);
  GPIO_Digital_Output(&GPIOE_BASE, 0xFF00);
  GPIO_Digital_Output(&GPIOB_BASE, _GPIO_PINMASK_4);
  GPIOB_ODR.B4 = 0;
  TFT_Set_Default_Mode();
  TFT_Set_MM_Plus();
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_SSD1963(480, 272);
  TFT_Set_DBC_SSD1963(255);
  
  // Init SDIO module
  SDIO_Init(_SDIO_CFG_POWER_SAVE_DISABLE | _SDIO_CFG_4_WIDE_BUS_MODE | _SDIO_CFG_CLOCK_BYPASS_DISABLE
            | _SDIO_CFG_CLOCK_RISING_EDGE | _SDIO_CFG_HW_FLOW_DISABLE, 125, &_GPIO_MODULE_SDIO_D0_D3);

  // Set pull-ups on SDIO lines
  GPIOD_PUPDR = GPIOD_PUPDR | 0x10;
  GPIOC_PUPDR = GPIOC_PUPDR | 0x550000;

  // The function sets used MMC interface to SDIO
  Mmc_Set_Interface(_MMC_INTERFACE_SDIO);
}
/*********************************************************************************************/
void DrawMP3Scr(){
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 460, 220);
  TFT_LIne(20,  46, 460,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("MP3 Player", 195, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Roman Toropov for STM32", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.ingineer.ru", 365, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}
/**************************************************************************************************
* Function MC3_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MP3 VS1053B codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/

void MP3_Init(void)
{
  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000);   // default 12 288 000 Hz

  // Maximum volume is 0x00 and total silence is 0xFE.
  volume_left  = 0; //0x3F;
  volume_right = 0; //0x3F;
  MP3_Set_Volume(volume_left, volume_right);
}

/**************************************************************************************************
* Function MP3_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI to communicate with MP3 codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Start(void)
{

  MP3_CS  = 1;               // Deselect MP3_CS
  MP3_RST = 1;               // Set MP3_RST pin

  BSYNC   = 0;               // Clear BSYNC
  BSYNC   = 1;               // Clear BSYNC

  // Initialize SPI2 module
  SPI2_Init_Advanced(_SPI_FPCLK_DIV64, _SPI_MASTER | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI2_PB13_14_15);

  TFT_BLED = 1;
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);

  TFT_Write_Text("1. Initializing VS1053B decoder interface", 30, 80);

  MP3_Init();
  Delay_ms(1000);
}

/**************************************************************************************************
* Function MP3_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the MP3 test
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void MP3_Test(char *test)
{
  unsigned long i;

  // Reset error flag
  *test = 0;

  TFT_Write_Text("2. Initializing MMC_FAT", 30, 100);
  if (Mmc_Fat_Init() == 0) {

    // Increase SDIO speed (SDIO_CK)
    SDIO_Init(_SDIO_CFG_POWER_SAVE_DISABLE | _SDIO_CFG_4_WIDE_BUS_MODE | _SDIO_CFG_CLOCK_BYPASS_DISABLE
            | _SDIO_CFG_CLOCK_RISING_EDGE | _SDIO_CFG_HW_FLOW_DISABLE, 1, &_GPIO_MODULE_SDIO_D0_D3);


    if (Mmc_Fat_Assign(&mp3_filename[roma], 0) ) {
      TFT_Write_Text("3. File Assigned", 30, 120);
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          //   procedure returns size of the file

      TFT_Write_Text("4. Play file: ", 30, 140);
      TFT_Write_Text(&mp3_filename[roma], 100, 140);
      // send file blocks to MP3 SDI
      while (file_size > BUFFER_SIZE)
      {
        for (i=0; i<BUFFER_SIZE; i++)
        {
          Mmc_Fat_Read(mp3_buffer + i);
        }
        for (i=0; i<BUFFER_SIZE/BYTES_2_WRITE; i++) {
          MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
        }

        file_size -= BUFFER_SIZE;
     }

      // send the rest of the file to MP3 SDI
      for (i=0; i<file_size; i++)
      {
        Mmc_Fat_Read(mp3_buffer + i);
      }

      for (i=0; i<file_size; i++)
      {
        MP3_SDI_Write(mp3_buffer[i]);
      }

      // TFT_Write_Text("\x20\x20\x20\x20\x20\x20\x20", 100, 140);
      TFT_Set_Pen(CL_WHITE, 12);
      TFT_H_Line(100, 150, 150);

    }
    else {

      TFT_Write_Text("3. File not assigned", 30, 120);
      *test = 2;
    }
  }
  else {
    TFT_Write_Text("3. MMC FAT not initialized", 30, 120);
    *test = 2;
  }
}

/**************************************************************************************************
* main function
**************************************************************************************************/

void main() {
  InitMCU();
  DrawMP3Scr();
  roma=0;
  MP3_Start();
  while(1){
     // &mp3_filename[roma];
     MP3_Test(&ucMP3_run_test);
     roma++;
     if (roma>24) { roma=0; }
  }
}